@@ -93,13 +93,14 @@ sub read
                 map {$self->{'val'}{$_} = $c++} ($first .. $last);
             }
         }
+        $self->{'count'} = $num;
     } elsif ($fmt == 1)
     {
         $fh->read($dat, 2);
         $first = $num;
         ($num) = unpack("n", $dat);
         $fh->read($dat, $num << 1);
-        map {$self->{'val'}{$first++} = $_} unpack("n*", $dat);
+        map {$self->{'val'}{$first++} = $_; $self->{'max'} = $_ if ($_ > $self->{'max'})} unpack("n*", $dat);
     } elsif ($fmt == 2)
     {
         $fh->read($dat, $num * 6);
@@ -107,6 +108,7 @@ sub read
         {
             ($first, $last, $c) = unpack("n3", substr($dat, $i * 6, 6));
             map {$self->{'val'}{$_} = $c} ($first .. $last);
+            $self->{'max'} = $c if ($c > $self->{'max'});
         }
     }
     $self;
@@ -134,15 +136,18 @@ sub out
         {
             $fmt = 2;
             last;
-        } elsif ($gids[$i] == $gids[$i-1] + 1)
+        } elsif ($gids[$i] == $gids[$i-1] + 1 && ($self->{'cover'} || $self->{'val'}{$gids[$i]} == $self->{'val'}{$gids[$i-1]}))
         { $eff++; }
         else
-        { $grp++; }
+        {
+            $grp++;
+            $eff += $gids[$i] - $gids[$i-1] if (!$self->{'cover'});
+        }
     }
-    if ($self->{'cover'})
-    { $fmt = 2 if ($eff / $grp > 4); }
-    else
-    { $fmt = 2 if ($grp > 1); }
+#    if ($self->{'cover'})
+    { $fmt = 2 if ($eff / $grp > 3); }
+#    else
+#    { $fmt = 2 if ($grp > 1); }
     
     if ($fmt == 1 && $self->{'cover'})
     {
@@ -156,7 +161,7 @@ sub out
         foreach $g (@gids)
         {
             if ($g > $last + 1)
-            { &$shipout(pack('n*', 0 x ($g - $last - 1))); }
+            { &$shipout(pack('n*', (0) x ($g - $last - 1))); }
             &$shipout(pack('n', $self->{'val'}{$g}));
             $last = $g;
         }
@@ -201,7 +206,7 @@ sub out
 }
 
 
-=head2 $c->add($glyphid)
+=head2 $c->add($glyphid[, $class])
 
 Adds a glyph id to the coverage table incrementing the count so that each subsequent addition
 has the next sequential number. Returns the index number of the glyphid added
@@ -210,14 +215,76 @@ has the next sequential number. Returns the index number of the glyphid added
 
 sub add
 {
-    my ($self, $gid) = @_;
+    my ($self, $gid, $class) = @_;
     
     return $self->{'val'}{$gid} if (defined $self->{'val'}{$gid});
-    $self->{'val'}{$gid} = $self->{'count'};
-    return $self->{'count'}++;
+    if ($self->{'cover'})
+    {
+        $self->{'val'}{$gid} = $self->{'count'};
+        return $self->{'count'}++;
+    }
+    else
+    {
+        $self->{'val'}{$gid} = $class || '0';
+        $self->{'max'} = $class if ($class > $self->{'max'});
+        return $class;
+    }
 }
 
 
+=head2 $c->signtaure
+
+Returns a vector of all the glyph ids covered by this coverage table or class
+
+=cut
+
+sub signature
+{
+    my ($self) = @_;
+    my ($vec, $range, $size);
+
+if (0)
+{
+    if ($self->{'cover'})
+    { $range = 1; $size = 1; }
+    else
+    {
+        $range = $self->{'max'};
+        $size = 1;
+        while ($range > 1)
+        {
+            $size = $size << 1;
+            $range = $range >> 1;
+        }
+        $range = $self->{'max'} + 1;
+    }
+    foreach (keys %{$self->{'val'}})
+    { vec($vec, $_, $size) = $self->{'val'}{$_} > $range ? $range : $self->{'val'}{$_}; }
+    length($vec) . ":" . $vec;
+}
+    $vec = join(";", map{"$_,$self->{'val'}{$_}"} keys %{$self->{'val'}});
+}
+
+=head2 @map=$c->sort
+
+Sorts the coverage table so that indexes are in ascending order of glyphid.
+Returns a map such that $map[$new_index]=$old_index.
+
+=cut
+
+sub sort
+{
+    my ($self) = @_;
+    my (@res, $i);
+
+    foreach (sort {$a <=> $b} keys %{$self->{'val'}})
+    {
+        push(@res, $self->{'val'}{$_});
+        $self->{'val'}{$_} = $i++;
+    }
+    @res;
+}
+
 =head2 $c->out_xml($context)
 
 Outputs this coverage/class in XML
